import jwt

from anilius.core.settings import settings


def encode_jwt(claim):
    return jwt.encode(claim, settings["JWT_SECRET"], algorithm="HS512")


def decode_jwt(encoded):
    return jwt.decode(encoded, settings["JWT_SECRET"], algorithms=["HS512"])



import uuid

from sqlalchemy import Column, DateTime
from sqlalchemy.sql import func

from anilius.core.db import Model, db_session
from anilius.model.uuid import UUID
from anilius.utils.uuid import validate_uuid4


class BaseModel(Model):
    __abstract__ = True

    id = Column(UUID(), primary_key=True, default=uuid.uuid4)
    created_on = Column(DateTime, default=func.now())
    updated_on = Column(DateTime, default=func.now(), onupdate=func.now())

    @classmethod
    def create(cls, commit=True, **entity_properties):
        entity = cls(**entity_properties)
        db_session().add(entity)

        if commit:
            db_session.commit()

        return entity

    @classmethod
    def get(cls, entity_id):
        if not validate_uuid4(entity_id):
            raise ValueError("entity id should be uuid4")

        entity = db_session().query(cls).get(entity_id)
        return entity

    @classmethod
    def list(cls, condition_dict):
        entity = db_session().query(cls).filter(*condition_dict).all()
        return entity



from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker

from anilius.core.settings import settings
from anilius.utils.singleton import Singleton


class DB(metaclass=Singleton):
    engine = None
    Model = None
    session = None

    def __init__(self):
        self.engine = create_engine(
            settings["DATABASE_URI"],
            convert_unicode=True,
            **settings["DATABASE_CONNECT_OPTIONS"]
        )
        self.session = scoped_session(
            sessionmaker(autocommit=False, autoflush=False, bind=self.engine)
        )
        self.Model = declarative_base(name="Model")
        self.Model.query = self.session.query_property()

    def get_model(self):
        return self.Model


DB = DB()
db_session = DB.session
Model = DB.get_model()



import io
import re

from setuptools import find_packages
from setuptools import setup

with io.open("README.md", "rt", encoding="utf8") as f:
    readme = f.read()

with io.open("src/anilius/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
    name='Anilius',
    version=version,
    description='GRPC framework for microservice ecosystem',
    long_description=readme,
    author='Mahdi Zarrintareh',
    author_email='mza2rintareh@gmail.com',
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        "alembic>=1.3.2",
        "black>=18.9b0",
        "coverage>=5.*",
        "flake8>=3.7.*",
        "grpcio>=1.26.*",
        "grpcio-tools>=1.26.*",
        "mysqlclient>=1.4.6",
        "passlib>=1.7.2",
        "phonenumbers>=8.11.1",
        "pytest>=5.3.4",
        "pytest-cov>=2.8.1",
        "pytest-xdist>=1.31.0",
        "PyJWT>=1.7.1",
        "py-grpc-prometheus>=0.2.0",
        "sentry-sdk>=0.14.1",
        "SQLAlchemy>=1.3.12",
        "SQLAlchemy-Utils>=0.36.1",
        "zimports>=0.2.0",
    ],
    extras_require={
        "dotenv": ["python-dotenv"],
        "dev": [
            "pytest",
            "coverage",
            "tox",
        ],
    },
    entry_points={"console_scripts": ["anilius = anilius.cli.main:main"]},
)



import os

import click
from anilius.cli.version import version_option


class AniliusGroup(click.Group):
    """Special subclass of the :class:`AppGroup` group that supports
    loading more commands from the configured Anilius app.  Normally a
    developer does not have to interface with this class but there are
    some very advanced use cases for which it makes sense to create an
    instance of this.
    For information as of why this is useful see :ref:`custom-scripts`.
    :param add_default_commands: if this is True then the default run and
        shell commands will be added.
    :param add_version_option: adds the ``--version`` option.
    :param create_app: an optional callback that is passed the script info and
        returns the loaded app.
    :param load_dotenv: Load the nearest :file:`.env` and :file:`.flaskenv`
        files to set environment variables. Will also change the working
        directory to the directory containing the first file found.
    :param set_debug_flag: Set the app's debug flag based on the active
        environment
    .. versionchanged:: 1.0
        If installed, python-dotenv will be used to load environment variables
        from :file:`.env` and :file:`.flaskenv` files.
    """

    def __init__(
        self,
        create_app=None,
        add_version_option=True,
        load_dotenv=True,
        set_debug_flag=True,
        **extra
    ):
        params = list(extra.pop("params", None) or ())

        if add_version_option:
            params.append(version_option)

        click.Group.__init__(self, params=params, **extra)
        self.create_app = create_app
        self.load_dotenv = load_dotenv
        self.set_debug_flag = set_debug_flag

        self._loaded_plugin_commands = False

    def main(self, *args, **kwargs):
        os.environ["ANILIUS_RUN_FROM_CLI"] = "true"

        kwargs.setdefault("auto_envvar_prefix", "ANILIUS")
        return super(AniliusGroup, self).main(*args, **kwargs)


cli = AniliusGroup(
    help="""\
A general utility script for Anilius microservice.
Provides commands from Anilius, extensions, and the application. Loads the
application defined in the ANILIUS_APP environment variable.
Setting the ANILIUS_ENV environment variable to 'development' will enable
debug mode.
\n
  {prefix}{cmd} ANILIUS_APP=hello.py\n
  {prefix}{cmd} ANILIUS_ENV=development\n
  {prefix}anilius run\n
""".format(
        cmd="export" if os.name == "posix" else "set",
        prefix="$ " if os.name == "posix" else "> ",
    )
)
